use std::fs::File;
use std::io::BufWriter;

extern crate ffmpeg_next as ffmpeg;
use ffmpeg::{software::scaling::flag::Flags, util::format::pixel::Pixel};

fn main() {
    let input_path = std::env::args().nth(1).expect("missing input file");
    let output_path = std::env::args().nth(2).expect("missing ouput file");

    ffmpeg::init().unwrap();
    ffmpeg::log::set_level(ffmpeg::log::Level::Info);

    // Open the file and dump some info about it to the console.
    let mut input_ctx = ffmpeg::format::input(&input_path).unwrap();
    ffmpeg::format::context::input::dump(&input_ctx, 0, Some(&input_path));

    // Get the "best" video stream.
    // (It's not clear to me what that means, but I'm going with it.)
    let stream = input_ctx
        .streams()
        .best(ffmpeg::media::Type::Video)
        .unwrap();
    let stream_idx = stream.index() as usize;

    // Create a decoder for the video stream.
    let mut decoder = ffmpeg::codec::context::Context::from_parameters(stream.parameters())
        .unwrap()
        .decoder()
        .video()
        .unwrap();

    // Create a converter to get the video frames into the color format
    // we want.
    // It's a bit weird, but we do this by creating a scaler, which also
    // resizes images.  Apparently it's an all-in-one kind of thing for
    // scaling and color format conversion.
    let mut converter = ffmpeg::software::scaling::context::Context::get(
        decoder.format(),
        decoder.width(),
        decoder.height(),
        Pixel::RGB24, // Output color format.
        decoder.width(),
        decoder.height(),
        Flags::POINT, // Nearest neighbor, since we're not actually scaling.
    )
    .unwrap();

    // Place to store the image data.
    let mut width = 0;
    let mut height = 0;
    let mut pixels = Vec::new();

    // Get the first frame of the video.
    // To access it we have to process packets until the frame is
    // available.
    let mut decoded_frame = ffmpeg::util::frame::video::Video::empty();
    for (stream, packet) in input_ctx.packets() {
        if stream.index() == stream_idx {
            decoder.send_packet(&packet).unwrap();
            if decoder.receive_frame(&mut decoded_frame).is_ok() {
                // Convert to RGB.
                let mut converted_frame = ffmpeg::util::frame::video::Video::empty();
                converter.run(&decoded_frame, &mut converted_frame).unwrap();

                // Save the image data to our own in-memory area.
                width = converted_frame.width() as u32;
                height = converted_frame.height() as u32;
                // The `0` parameter is the plane.  For this pixel format
                // there's just one plane anyway.
                for byte in converted_frame.data(0).iter() {
                    pixels.push(*byte);
                }

                // We got the first frame, so stop.
                break;
            }
        }
    }

    // Write out a PNG file.
    let mut encoder = png::Encoder::new(
        BufWriter::new(File::create(output_path).unwrap()),
        width,
        height,
    );
    encoder.set_color(png::ColorType::Rgb);
    encoder.set_depth(png::BitDepth::Eight);
    let mut writer = encoder.write_header().unwrap();
    writer.write_image_data(&pixels).unwrap();

    println!("{:?}", decoder.format());
}

// Returns the number of bits needed to store any of the pixel channels
// for a given format.
fn pixel_channel_size(f: Pixel) -> usize {
    use Pixel::*;
    match f {
        YUV420P => 8,
        YUV422P10LE => 10,
        _ => panic!("Haven't decided how to decode '{:?}' yet", f),
    }
}
